--[[ Common ]]

--const
local MIN_VISIBLE_BUTTONS_SETTINGS_MODE = 1
FBC_BUTTON_PLACE_SIZE 	= 36
FBC_BUTTON_PLACE_OFFSET	= 4
FBC_FRAME_OFFSET 		= 5
FBC_ARROW_OFFSET 		= 4

--arrow button directions
FBC_DIR_UP		= 1
FBC_DIR_LEFT	= 2
FBC_DIR_DOWN	= 3
FBC_DIR_RIGHT	= 4

local SpellCacheToSlot = {}
local SpellCacheToId = {}

local function GetFullSpellName(name, rank)
	if not(rank) then
		rank = ""
	end
	if (name) then
		return name.."("..rank..")", name
	end
	return nil, nil
end

local function GetSpellNameBySlot(slot, book)
	--local item_type, id = GetSpellBookItemInfo(slot, BOOKTYPE_SPELL)
	--print("item_type "..tostring(item_type)..", id "..tostring(id))
	local name, rank = GetSpellBookItemName(slot, book)
	return GetFullSpellName(name, rank)
end

local function CacheSpellsByType(book, skillType)
	local slot = 1
	local cache2slot = {}
	local cache2id = {}
	while true do
		local item_type, id = GetSpellBookItemInfo(slot, book)
		if not(item_type) then
			break
		end
		if (item_type == skillType) then
			local nameRank, rank = GetSpellNameBySlot(slot, book)
			if not(nameRank) then
				break
			end
			cache2slot[nameRank] = slot
			cache2id[nameRank] = id
		end
		slot = slot + 1
	end
	return cache2slot, cache2id
end

local SpellTypesToCache = {
	{
		["book"] = BOOKTYPE_SPELL,
		["skillType"] = "SPELL",
	}, -- [1]
	{
		["book"] = BOOKTYPE_PET,
		["skillType"] = "PETACTION",
	}, -- [2]
}

local CompanionTypes = {"MOUNT", "CRITTER"}

local function CacheCompanionSpells(compType)
	local cache2slot = {}
	local cache2id = {}
	local name, id
	for slot = 1, GetNumCompanions(compType) do
		--creatureID, creatureName, creatureSpellID, icon, issummoned
		_, name, id = GetCompanionInfo(compType, slot)
		local spellName = GetSpellInfo(id)
		--print("compType "..compType..", name "..name..", id "..id..", slot "..slot)
		cache2slot[spellName] = slot
		cache2id[spellName] = id
	end
	return cache2slot, cache2id
end

function FlyoutButton_CacheSpells(BOOKTYPE_SPELL, BOOKTYPE_PET)
	for k, v in ipairs(SpellTypesToCache) do
		SpellCacheToSlot[v.book], SpellCacheToId[v.book] = CacheSpellsByType(v.book, v.skillType) 
	end
	for _, v in pairs(CompanionTypes) do
		SpellCacheToSlot[v], SpellCacheToId[v] = CacheCompanionSpells(v) 
	end
end

function FlyoutButton_FindSpellSlot(spellName, book)
	if SpellCacheToSlot[book] then
		return SpellCacheToSlot[book][spellName]
	end
	return nil
end

function FlyoutButton_FindSpellId(spellName, book)
	--print("spellName "..tostring(spellName)..", book "..tostring(book))
	if SpellCacheToId[book] then
		return SpellCacheToId[book][spellName]
	end
	return nil
end

function FlyoutButton_GetCompanionInfoByName(name, compType)
	if SpellCacheToId[compType] then
		--print("name "..name..", compType "..compType)
		local slot = SpellCacheToSlot[compType][name]
		if slot then
			--creatureID, creatureName, creatureSpellID, icon, issummoned
			return GetCompanionInfo(compType, slot)
		end
	end
	return nil
end

function FlyoutButton_SetCursor(command, value, subValue)
	--print("SetCursor "..tostring(command)..", "..tostring(value)..", "..tostring(subValue))
	if not(command) then
		return
	end

	ClearCursor()
	if (command == "spell") then
		local slot = FlyoutButton_FindSpellSlot(value, subValue)
		if slot then
			if (subValue == "MOUNT" or subValue == "CRITTER") then
				PickupCompanion(subValue, slot)
			else
				PickupSpellBookItem(slot, subValue)
			end
		end
	elseif (command == "item") then
		PickupItem(value)	--itemID or "itemString" or "itemName" or "itemLink"
	elseif (command == "macro") then
		PickupMacro(value)
	elseif (command == "equipmentset") then
		PickupEquipmentSetByName(value)
	end
end

function FlyoutButton_GetCursorValues()
	local command, value, subValue, spellId = GetCursorInfo()
	--print("GetCursorValues "..tostring(command)..", "..tostring(value)..", "..tostring(subValue)..", "..tostring(spellId))
	if (command == "spell") then
		value = GetSpellNameBySlot(value, subValue)
		--print("value "..tostring(value))
	elseif (command == "item") or (command == "equipmentset") then
		--nothing to do
	elseif (command == "macro") then		
		subValue = value -- value is macro index	
		value = GetMacroInfo(value) --name, texture, body
	elseif (command == "companion") then
		local _, _, spellId = GetCompanionInfo(subValue, value)
		value = GetSpellInfo(spellId)
		command = "spell"
	end
	return command, value, subValue
end

function FlyoutButton_GetListButtonsCount(listTable)
	local count = 0
	if listTable then
		for i = 1, #listTable do
			if listTable[i].value then
				count = i
			end
		end
	end
	local settingsCount = count
	
	if FbcSettingsMode then
		if count < MIN_VISIBLE_BUTTONS_SETTINGS_MODE then
			settingsCount = MIN_VISIBLE_BUTTONS_SETTINGS_MODE
		else
			settingsCount = count + 1
		end
	end
	return count, settingsCount
end

